@@ -6,7 +6,7 @@ import functools
 import hashlib
 import operator
 import re
-from typing import TYPE_CHECKING, Any, List, Tuple, Union
+from typing import TYPE_CHECKING, Any, Generator, List, Tuple, Union
 
 from django.db import DEFAULT_DB_ALIAS, models, router, transaction
 from django.db.models.expressions import Col
@@ -55,6 +55,9 @@ class _Primitive:
     def __str__(self):
         return self.name
 
+    def __hash__(self):
+        return hash(self.name)
+
 
 class Level(_Primitive):
     values = ("ROW", "STATEMENT")
@@ -83,6 +86,12 @@ class Referencing:
         self.old = old
         self.new = new
 
+    def __eq__(self, other):
+        if not isinstance(other, Referencing):  # pragma: no cover
+            return False
+
+        return self.old == other.old and self.new == other.new
+
     def __str__(self):
         ref = "REFERENCING"
         if self.old:
@@ -121,6 +130,15 @@ class Operation(_Primitive):
         assert isinstance(other, Operation)
         return Operations(self, other)
 
+    def __contains__(self, other):  # pragma: no cover
+        return self == other
+
+    def __iter__(self):  # pragma: no cover
+        return iter([self])
+
+    def __str__(self):
+        return self.name
+
 
 class Operations(Operation):
     """For providing multiple operations `OR`ed together.
@@ -134,9 +152,19 @@ class Operations(Operation):
 
         self.operations = operations
 
+    def __or__(self, other):
+        assert isinstance(other, Operation)
+        return Operations(*self.operations, other)
+
     def __str__(self):
         return " OR ".join(str(operation) for operation in self.operations)
 
+    def __contains__(self, other):  # pragma: no cover
+        return other in self.operations
+
+    def __iter__(self):  # pragma: no cover
+        return iter(self.operations)
+
 
 Update = Operation("UPDATE")
 """
@@ -343,6 +371,25 @@ class Q(models.Q, Condition):
         def __invert__(self) -> Self: ...
 
 
+def _normalize_fields(model: type[models.Model], fields: list[str]) -> Generator[str, None, None]:
+    """Normalize and validate fields.
+
+    Fields such as "my_foreign_key_id" will be normalized to the name on the model, such
+    as "my_foreign_key".
+
+    Fields such as M2Ms are not allowed in triggers, so they raised
+    errors here.
+    """
+    fields = [model._meta.get_field(field).name for field in fields]
+
+    for field in fields:
+        dj_field = model._meta.get_field(field)
+        if hasattr(dj_field, "m2m_db_table"):
+            raise ValueError(f'Cannot filter on many-to-many field "{field}".')
+
+        yield dj_field.name
+
+
 class _Change(Condition):
     """For specifying a condition based on changes to fields.
 
@@ -374,18 +421,16 @@ class _Change(Condition):
         return inverted
 
     def resolve(self, model: type[models.Model]) -> str:
+        fields = list(_normalize_fields(model, self.fields))
+        exclude = set(_normalize_fields(model, self.exclude))
         model_fields = {f.name for f in model._meta.fields}
-        for field in self.fields + self.exclude:
-            if field not in model_fields:
-                raise ValueError(f'Field "{field}" not found on model "{model}"')
 
-        exclude = set(self.exclude)
         if self.exclude_auto:
             for f in model._meta.fields:
                 if getattr(f, "auto_now", False) or getattr(f, "auto_now_add", False):
                     exclude.add(f.name)
 
-        fields = sorted(f for f in (self.fields or model_fields) if f not in exclude)
+        fields = sorted(f for f in (fields or model_fields) if f not in exclude)
 
         if set(fields) == model_fields and not self.all:
             if self.comparison == "df":
@@ -512,19 +557,17 @@ class Func:
     def __init__(self, func):
         self.func = func
 
-    def render(self, model: models.Model) -> str:
+    def render(self, **kwargs: Any) -> str:
         """
         Render the SQL of the function.
 
         Args:
-            model: The model.
+            **kwargs: Keyword arguments to pass to the function template.
 
         Returns:
             The rendered SQL.
         """
-        fields = utils.AttrDict({field.name: field for field in model._meta.fields})
-        columns = utils.AttrDict({field.name: field.column for field in model._meta.fields})
-        return self.func.format(meta=model._meta, fields=fields, columns=columns)
+        return self.func.format(**kwargs)
 
 
 # Allows Trigger methods to be used as context managers, mostly for
@@ -752,6 +795,15 @@ class Trigger:
         """
         return f"{self.get_pgid(model)}()"
 
+    def get_func_template_kwargs(self, model: models.Model) -> dict[str, Any]:
+        """
+        Returns a dictionary of keyword arguments to pass to the function template
+        when using Func() classes.
+        """
+        fields = utils.AttrDict({field.name: field for field in model._meta.fields})
+        columns = utils.AttrDict({field.name: field.column for field in model._meta.fields})
+        return {"meta": model._meta, "fields": fields, "columns": columns}
+
     def render_func(self, model: models.Model) -> str:
         """
         Renders the func.
@@ -765,7 +817,7 @@ class Trigger:
         func = self.get_func(model)
 
         if isinstance(func, Func):
-            return func.render(model)
+            return func.render(**self.get_func_template_kwargs(model))
         else:
             return func
 
